/* icmplog.c of iplog v1.8 -- eric@ojnk.org */
#include "config.h"
#include "common.h"
#include "icmp-compat.h"
#include "icmplog.h"

int main(void)
{
	int sock;
	struct icmp_packet pkt;

#ifdef DETECT_SMURF
	signal(SIGALRM, sigalrm);
#endif
	signal(SIGHUP, SIG_IGN);

	if ((sock = socket(AF_INET, SOCK_RAW, IPPROTO_ICMP)) < 0) {
		perror("socket");
		exit(EXIT_FAILURE);
	}
	openlog("icmplog", LOGGING_OPTIONS, LOGGING_FACILITY);
	signal(SIGHUP, SIG_IGN);

	fork_to_background();
	initialize_lockfile("icmplog");
	drop_privileges();

	for (;;) {
#ifdef DETECT_SMURF
		errno = 0;
		read(sock, &pkt, sizeof(pkt));
		if (errno == EINTR)
			continue;
#else
		read(sock, &pkt, sizeof(pkt));
#endif

		if (pkt.ip.ip_hl != 5) {
			syslog(LEVEL, "suspicious ip options from %s", inet_ntoa(pkt.ip.ip_src));
			continue;
		}
		switch (MIN(pkt.icmp.icmp_type, 19)) {
			case ICMP_TIME_EXCEEDED:
				break;			/* ignored */
			case ICMP_ECHO_REPLY:
#ifdef DETECT_SMURF
				if (smurf(&pkt.ip))
					continue;

				syslog(LEVEL, codes[(int)pkt.icmp.icmp_type], inet_ntoa(pkt.ip.ip_src), UNFIX(pkt.ip.ip_len) - ICMP_SIZE);
				break;
#endif
			case ICMP_ECHO:
				syslog(LEVEL, codes[(int)pkt.icmp.icmp_type], inet_ntoa(pkt.ip.ip_src), UNFIX(pkt.ip.ip_len) - ICMP_SIZE);
				break;
			case ICMP_DEST_UNREACHABLE:
				if (pkt.icmp.icmp_code > UNREACHABLE_MAX) {
#ifdef LOG_WEIRD_ICMP
					syslog(LEVEL, "undefined destination unreachable code from %s(?)", inet_ntoa(pkt.ip.ip_src));
#endif
					break;
				}
				syslog(LEVEL, unreach[(int)pkt.icmp.icmp_code], inet_ntoa(pkt.ip.ip_src), protolookup(pkt.icmp.icmp_ip.ip_p), UNFIX(pkt.data.th_dport), UNFIX(pkt.data.th_sport));
				break;
			case ICMP_REDIRECT:
				if (pkt.icmp.icmp_code > REDIRECT_MAX) {
#ifdef LOG_WEIRD_ICMP
					syslog(LEVEL, "undefined redirect code from %s(?)", inet_ntoa(pkt.ip.ip_src));
#endif
					break;
				}
				print_redirect(&pkt);
				break;
			case ICMP_TIMESTAMP_REPLY:
				print_timestamp_reply(&pkt);
				break;
			case ICMP_ADDRESS_REPLY:
				print_address_reply(&pkt);
				break;
			case ICMP_PARAMETER_PROBLEM:
				syslog(LEVEL, codes[(int)pkt.icmp.icmp_type], inet_ntoa(pkt.ip.ip_src), pkt.icmp.icmp_pptr);
				break;
			case ICMP_ROUTER_ADVERT:
				print_router_advertisement(&pkt);
				break;
			case ICMP_UNDEFINED:
#ifdef LOG_WEIRD_ICMP
				syslog(LEVEL, "undefined icmp type %d from %s(?)", (int)pkt.icmp.icmp_type, inet_ntoa(pkt.ip.ip_src));
#endif
				break;
			default:
				if (codes[(int)pkt.icmp.icmp_type])
					syslog(LEVEL, codes[(int)pkt.icmp.icmp_type], inet_ntoa(pkt.ip.ip_src));
#ifdef LOG_WEIRD_ICMP
				else
					syslog(LEVEL, "undefined icmp type %d from %s(?)", (int)pkt.icmp.icmp_type, inet_ntoa(pkt.ip.ip_src));
#endif
				break;
		}
	}
	(void)close(sock);
}

static char *protolookup(u_char proto)
{
	struct protoent *pe;

	if ((pe = getprotobynumber(proto)) != NULL)
		return (pe->p_name);
	else {
		static char buff[12];

		sprintf(buff, "proto %u", proto);
		return (buff);
	}
}

static void print_redirect(struct icmp_packet *pkt)
{
	char gate[18], saddr[18], daddr[18];

	strcpy(gate, inet_ntoa(pkt->icmp.icmp_gwaddr));
	strcpy(saddr, inet_ntoa(pkt->ip.ip_src));
	strcpy(daddr, inet_ntoa(pkt->icmp.icmp_ip.ip_dst));

	syslog(LEVEL, redir[(int)pkt->icmp.icmp_code], gate, saddr, daddr);
}

static void print_address_reply(struct icmp_packet *pkt)
{
	char saddr[18], daddr[18];
	struct in_addr in;

	in.s_addr = htonl(pkt->icmp.icmp_mask);
	strcpy(saddr, inet_ntoa(pkt->ip.ip_src));
	strcpy(daddr, inet_ntoa(in));

	syslog(LEVEL, codes[(int)pkt->icmp.icmp_type], saddr, daddr);
}


static void print_router_advertisement(struct icmp_packet *pkt)
{
	struct id_rdiscovery *routers;
	char msg[768], temp[64], duration[32];
	u_char n;
	u_int time = UNFIX(pkt->icmp.icmp_lifetime);

	memset(msg, 0, sizeof(msg));
	routers = (struct id_rdiscovery *)&pkt->icmp.icmp_dun.id_rdiscovery;

	for (n = 1; n <= MIN(pkt->icmp.icmp_num_addr, 12); n++, routers++) {
		memset(temp, 0, sizeof(temp));
		snprintf(temp, sizeof(temp), "%s%s", (n > 1) ? " -> " : "", inet_ntoa(routers->router_addr));
		snprintf(msg, sizeof(msg), "%s%s", msg, temp);
	}

	if (time < 60)
		snprintf(duration, sizeof(duration), "%u sec", time);
	else
		snprintf(duration, sizeof(duration), "%u min, %u sec", (time / 60), (time % 60));

	syslog(LEVEL, "path of router advertisement from %s: %s (lifetime %s)", inet_ntoa(pkt->ip.ip_src), msg, duration);
}

static void print_timestamp_reply(struct icmp_packet *pkt)
{
	char ret[16];
	struct tm *tm;
	u_long actual_time;

	actual_time = ntohl(pkt->icmp.icmp_ttime) / 1000;
	tm = localtime((time_t *) & actual_time);
	snprintf(ret, sizeof(ret), "%.2d:%.02d:%.02d", tm->tm_hour, tm->tm_min, tm->tm_sec);
	syslog(LEVEL, codes[pkt->icmp.icmp_type], inet_ntoa(pkt->ip.ip_src), ret);
	free(tm);
}
