/* iplog.h of iplog v1.8 -- eric@ojnk.org */
#include <stdarg.h>
#include <pwd.h>
#include <grp.h>
#include <sys/stat.h>

#include "config.h"
#include "common.h"

int comp(const void *a, const void *b)
{
	return *(int *)a - *(int *)b;
}

char *service_lookup(u_short port, const char *proto)
{
	struct servent *se;

	if ((se = getservbyport(port, proto)) != NULL)
		return (se->s_name);
	else {
		static char buff[12];

		sprintf(buff, "port %u", UNFIX(port));
		return (buff);
	}
}

u_short network_byte_ordering(u_short x)
{
#if BYTE_ORDER == LITTLE_ENDIAN
	u_char *s = (u_char *) & x;

	return (u_short) (s[0] << 8 | s[1]);
#else
	return x;
#endif
}

void fatal(const char *fmt,...)
{
	char p[512];
	va_list ap;

	va_start(ap, fmt);
	(void)vsnprintf(p, sizeof(p), fmt, ap);
	va_end(p);

	syslog(LOG_WARNING, p);

	exit(-1);
}

void fork_to_background(void)
{
	switch (fork()) {
		case -1:
			_exit(0);
		case 0:
			break;
		default:
			_exit(0);
	}

	if (setsid() == -1)
		fatal("setsid: %m");

	chdir("/");
	umask(022);

	(void)close(fileno(stdin));
	(void)close(fileno(stdout));
	(void)close(fileno(stderr));
}

void assert_id(uid_t uid, gid_t gid)
{
	struct passwd *pw;
	struct group *gr;
	uid_t my_uid;
	gid_t my_gid;

	if ((pw = getpwnam(DROP_USER)) == NULL) {
		if ((pw = getpwnam("nobody")) != NULL) {
			my_uid = pw->pw_uid;
		} else {
			my_uid = 65534;
		}
	} else
		my_uid = pw->pw_uid;

	if ((gr = getgrnam(DROP_GROUP)) != NULL)
		my_gid = gr->gr_gid;
	else if (((gr = getgrnam("nobody")) != NULL) || ((gr = getgrnam("nogroup")) != NULL))
		my_gid = gr->gr_gid;
	else if (pw != NULL)
		my_gid = pw->pw_gid;
	else
		my_gid = 65534;

	uid = my_uid;
	gid = my_gid;
}

void drop_privileges(void)
{
	gid_t gid = 65534;
	uid_t uid = 65534;

	assert_id(uid, gid);

	if (setregid(gid, gid) < 0)
		fatal("setregid: %m");

	if (setgroups(1, &gid) < 0)
		fatal("setgroups: %m");

	if (setreuid(uid, uid) < 0)
		fatal("setreuid: %m");
}

#ifdef LOGFILE
#undef openlog
#undef syslog
#define LOG_SIZE 1024
int log_fd, times = 0;
char *last_message = "x";
time_t last_time = 0;

void openlog(const char *name, int a, int b)
{
	char *log_name;

	a |= ~b;					/* make gcc shutup */

	log_name = malloc(strlen(LOGFILE) + strlen(name) + 1);
	if (log_name == NULL)
		fatal("openlog: %m");

	sprintf(log_name, LOGFILE, name);

	fprintf(stdout, "logging to file %s\n", log_name);

	if ((log_fd = open(log_name, O_WRONLY | O_CREAT | O_APPEND, 0644)) >= 0)
		syslog(LEVEL, "logging began");

	if (log_name != NULL)
		free(log_name);
}

void syslog(int unused, const char *fmt,...)
{
	time_t now;
	char tmp[LOG_SIZE];
	char *p = malloc(LOG_SIZE);
	va_list ap;

	(void)unused;				/* make gcc shutup */

	if (p == NULL)
		fatal("syslog: %m");

	va_start(ap, fmt);
	(void)vsnprintf(p, LOG_SIZE, fmt, ap);
	va_end(ap);

	time(&now);
	memset(&tmp, 0, sizeof(tmp));

	if (!strcmp(p, last_message)) {
		if ((last_time + 60) < now) {
			snprintf(tmp, LOG_SIZE, "%.15s last message repeated %d times\n", ctime(&now) + 4, times);
			write(log_fd, tmp, strlen(tmp));
			times = 0;
			time(&last_time);
		}
		times++;
		free(p);
		return;
	} else if (times > 0) {
		snprintf(tmp, LOG_SIZE, "%.15s last message repeated %d times\n", ctime(&now) + 4, times);
		write(log_fd, tmp, strlen(tmp));
		times = 0;
	}
	snprintf(tmp, LOG_SIZE, "%.15s %s\n", ctime(&now) + 4, p);
	write(log_fd, tmp, strlen(tmp));
	time(&last_time);
	last_message = strdup(p);

	free(p);
}
#endif
